# -*- perl -*-

# Copyright (c) 2002 by Jeff Weisberg
# Author: Jeff Weisberg <argus @ tcp4me.com>
# Date: 2002-Apr-03 08:56 (EST)
# Function: the group class
#
# $Id: Group.pm,v 1.13 2002/10/20 22:32:13 jaw Exp jaw $


package Group;
@ISA = qw(MonEl);

# And then, perchance because his breath was failing,
# He grouped himself together with a bush.
#   -- Dante, Divine Comedy

$doc = {
    package => __PACKAGE__,
    file    => __FILE__,
    isa     => [@ISA],
    
    methods => {

    },
    fields => {

    },
};


sub new {
    my $class = shift || 'Group';
    my $x = {};
    bless $x, $class;
    $x;
}

sub config {
    my $me = shift;
    my $cf = shift;
    
    $me->init_from_config( $doc, 'group' );
    return 0 unless $me->init( $cf );
}

sub readconfig {
    my $cf = shift;
    my $mom = shift;
    my $me = new;
    my( $line, $type, $name, $x, $nomoredata, @kinder );
    
    $me->{parents} = [ $mom ] if $mom;

    $line = $cf->nextline();
    ($type, $name) = $line =~ /(Group|Host)\s+\"(.*)\"/i;
    $me->{type} = $type;
    $me->{name} = $name;
    $me->{definedinfile} = $cf->{file};
    $me->{definedonline} = $cf->{line};

    while( defined($_ = $cf->nextline()) ){
	print STDERR "readgroup: $_\n" if $::opt_d;
	if( /^\s*\}/ ){
	    $me->{children} = [ @kinder ];
	    $me->config($cf);
	    return $me;
	}

	elsif( /^(Group|Host)\s/i ){
	    $nomoredata = 1;
	    $cf->ungetline($_);
	    $x = Group::readconfig($cf, $me);
	    return $x unless $x;
	    push @kinder, $x;
	}

	elsif( /^Alias\s/i ){
	    $nomoredata = 1;
	    $cf->ungetline($_);
	    $x = Alias::readconfig($cf, $me);
	    return $x unless $x;
	    push @kinder, $x;
	}

	elsif( /^Service/i ){	# no, not Service\s
	    $nomoredata = 1;
	    $cf->ungetline($_);
	    $x = Service::readconfig($cf, $me);
	    return $x unless $x;
	    push @kinder, $x;
	}

	elsif( /:/ ){
	    my($k, $v) = split /:\s*/, $_, 2;
	    # data must be before Service|Group|Alias 
	    # disallow
	    # Group "foo" {
	    #    x1: value
	    #    Service TCP/FOO
	    #    x2: value          <- error
	    # }
	    return $cf->error( "additional data not permitted here" )
		if $nomoredata;
	    $cf->warning( "redefinition of parameter '$k'" )
		if defined $me->{config}{$k};
	    $me->{config}{$k} = $v;
	}

	else{
	    return $cf->error( "invalid entry in config file: '$_'" );
	}
    }

    $me;
}

sub gen_conf {
    my $me = shift;
    my $in = shift;
    my( $k, $c, $r, $rc, $t );

    if( $me->{i_am_top} ){
	foreach $k (sort keys %Conf::Config){
	    $r .= "$k:\t$Conf::Config{$k}\n"
		unless( ($k =~ /^_/) && $Conf::Config{_hide_expr} );
	}
	foreach $c (@{$me->{children}}){
	    $r .= "\n";
	    $r .= $c->gen_conf($in);
	}
	return $r;
    }
    
    $r = "$me->{type} \"$me->{name}\" {\n";
    foreach $k (sort keys %{$me->{config}}){
	$r .= "\t$k:\t$me->{config}{$k}";
	$r .= "\t# unused parameter - typo?"
	    unless $me->{confck}{$k};
	$r .= "\n";
    }
    foreach $c (@{$me->{children}}){
	$rc .= $c->gen_conf($in + 1);
    }
    $rc =~ s/^/\t/gm;
    $r .= "$rc}\n";

    $r;
}

################################################################
# override MonEl virtual methods
################################################################
sub webpage {
    my $me = shift;
    my $fh = shift;
    my $topp = shift;
    my( $k, $v, $kk, $c, $x, %cs );

    print $fh "<!-- start of Group::webpage -->\n";
    
    # object data
    unless( $topp ){
	print $fh "<TABLE>\n";
	foreach $k (qw(name ovstatus flags info note comment annotation details)){
	    $v = $me->{$k};
	    $kk = $k;
	    $x = '';
	    if( $k eq 'ovstatus' ){
		$x = " BGCOLOR=\"" . MonEl::web_color($v) . "\"";
		$kk = 'status';
	    }
	    print $fh "<TR><TD>$kk</TD><TD$x>$v</TD></TR>\n" if defined($v);
	}
	print $fh "</TABLE>\n";

	$me->web_override($fh);
	print $fh "<HR>\n";
    }
    
    # children status
    if( @{$me->{children}} ){
	print $fh "<TABLE>\n";
	if( $topp ){
	    print $fh "<TR><TH>Name</TH><TH>Up</TH><TH>Down</TH><TH>Override</TH></TR>\n";
	    foreach $c (@{$me->{children}}){
		$c->web_page_row_top($fh);
	    }
	}else{
	    print $fh "<TR><TH>Name</TH><TH COLSPAN=3>Status</TH></TR>\n";
	    foreach $c (@{$me->{children}}){
		$c->web_page_row_base($fh);
	    }
	}
	print $fh "</TABLE>\n";
    }
    else{
	print $fh "<!-- no children -->\n";
    }
    
    print $fh "<!-- end of Group::webpage -->\n";
}


sub web_page_row_base {
    my $me = shift;
    my $fh = shift;
    my $label = shift;
    my( $kov, $color, $c, $s, %cs );

    foreach $c (@{$me->{children}}){
	my $cc = $c;
	$cc = $c->aliaslookup() if $c->can('aliaslookup');
	$kov = 1 if $c->{override};
	
	push @{$cs{ $cc->{ovstatus} }},
	    "<A HREF=\"". $c->url('func=page'). "\"><FONT COLOR=\"#000000\">".
		($c->{label_right}||$c->{name}).
		"</FONT></A>"
    }

    if( $me->{override} ){
	$color = ' BGCOLOR=' . MonEl::web_color('override');
    }
    print $fh "<TR><TD$color><A HREF=\"", $me->url('func=page'), "\">",
        ($label||$me->{label_left}||$me->{name}), "</A></TD>";

    foreach $s ('up', 'down', 'override'){
	my $cx = join( " &nbsp; ", @{$cs{$s}});
	if( $cx ){
	    print $fh "<TD BGCOLOR=\"", MonEl::web_color($s), "\">$cx</TD>\n";
	}else{
	    print $fh "<TD></TD>";
	}
    }
    print $fh "</TR>\n";
}

################################################################

Doc::register( $doc );
1;
