#!/usr/local/bin/perl
# -*- perl -*-

# Copyright (c) 2002 by Jeff Weisberg
# Author: Jeff Weisberg <argus @ tcp4me.com>
# Date: 2002-Apr-03 18:59 (EST)
# Function: testing of SNMP things
#
# $Id: SNMP.pm,v 1.8 2002/10/20 22:32:15 jaw Exp jaw $

package UDP;
use BER;

my $snmpid = rand( 1000000 );
sub get_request  { 0 | context_flag };

my %oids =
(
 AdminStatus   => '.1.3.6.1.2.1.2.2.1.7',
 OperStatus    => '.1.3.6.1.2.1.2.2.1.8',
 ifInErrors    => '1.3.6.1.2.1.2.2.1.14',
 ifOutErrors   => '1.3.6.1.2.1.2.2.1.20',
 ifInOctets    => '.1.3.6.1.2.1.2.2.1.10',
 ifOutOctets   => '.1.3.6.1.2.1.2.2.1.16',
 BGPPeerStatus => '.1.3.6.1.2.1.15.3.1.2',
 ciscoEnvMonTemperatureStatusValue => '.1.3.6.1.4.1.9.9.13.1.3.1.3',
 # ...
 
 );

sub snmp_init {
    my $me = shift;
    my $cf = shift;
    my( $req, $oid );

    $oid = $me->{snmp}{oid};

    return $cf->error( "invalid OID ($oid)" )
	unless $oid;

    $oid =~ s/([^\d\.]+)/$oids{$1}||'UNKNOWN'/ge;
    
    return $cf->error( "invalid OID ($oid)" )
	unless( $oid =~ /^[\d\.]+$/ );

    # Why lifts she up her arms in sequence thus?
    #   -- Shakespeare, Titus Andronicus
    # SNMP v1 packet
    $req = encode_sequence( encode_int(0),
		    encode_string( $me->{snmp}{community} ), 
		    encode_tagged_sequence(
				   get_request,
				   encode_int( $snmpid++ ),
				   encode_int_0,
				   encode_int_0,
				   encode_sequence(
					   encode_sequence(
						   encode_oid(split /\./, $oid ),
						   encode_null()))) );
    $me->{udp}{send} = $req;
    $me->{udp}{port} = $me->{config}{port} || 161;
    $me->{udp}{decodesnmp} = 1;
    $me;
}

sub snmp_recv {
    my $me = shift;
    my $l  = shift;
    my( $foo, $transient );

    $BER::errmsg = '';
    my( $ver, $comm, $reqid, $errst, $errin, $oid, $value ) =
	decode_by_template($l, "%{%i%s%*{%i%i%i%{%{%O%@", 162 );

    $value = pretty_print($value);

    if( $BER::errmsg ){
	return $me->isdown( "BER error: $BER::errmsg" );
    }

    if( $value eq "(null)" ){
	return $me->isdown( "BER error: NULL (no such OID?)" );
    }
    
    $me->debug( "SNMP recv - raw value $value" );
    
    if( $me->{snmp}{snmpcalc} && $me->{snmp}{snmpcalc} =~ /rate/ ){
	$value *= 8;	# convert octets -> bits
	
	if( $me->{snmp}{snmpcalcalcdata}{lastt} && $me->{snmp}{snmpcalcdata}{lastt} != $^T ){
	    $foo = ($value - $me->{snmp}{snmpcalcdata}{lastv})
		/ ($^T - $me->{snmp}{snmpcalcdata}{lastt});
	}else{
	    $transient = 1;
	    $foo = 0;
	}
	$me->{snmp}{snmpcalcdata}{lastv} = $value;
	$me->{snmp}{snmpcalcdata}{lastt} = $^T;
	$value = $foo;
    }

    # Why birds and beasts from quality and kind,
    # Why old men fool and children calculate,
    #   -- Shakespeare, Julius Ceasar
    if( $me->{snmp}{snmpcalc} && $me->{snmp}{snmpcalc} =~ /ave/ ){
	# moving average
	my $x = $me->{snmp}{snmpcalcdata}{ave};
	$x += $value;
	$x /= 2;
	$me->{snmp}{snmpcalcdata}{ave} = $x;
	$value = $x;
    }
    
    if( $me->{snmp}{snmpcalc} ){
	$value = sprintf "%.2f", $value;
    }
    
    $me->{snmp}{currvalue} = $value;
    $me->{srvc}{result} = $value;
    $me->debug( "SNMP recv - curr $value" );

    # Loud peals of thunder from the poles ensue; 
    # Then flashing fires the transient light renew;
    #   -- Virgil, Aeneid
    return $me->done() if $transient;
    
    # test value
    # Lesser than Macbeth, and greater.
    #   -- Shakespeare, Macbeth
    if( defined($me->{snmp}{minvalue}) && $value < $me->{snmp}{minvalue} ){
	return $me->isdown( "SNMP less than min" );
    }
    # The greater scorns the lesser
    #   -- Shakespeare, Timon of Athens
    if( defined($me->{snmp}{maxvalue}) && $value > $me->{snmp}{maxvalue} ){
	return $me->isdown( "SNMP more than max" );
    }
    # Repugnant to command: unequal match'd
    #   -- Shakespeare, Hamlet
    if( defined($me->{snmp}{eqvalue}) && $value != $me->{snmp}{eqvalue} ){
	return $me->isdown( "SNMP not equal" );
    }
    # In equal scale weighing delight and dole
    #   -- Shakespeare, Hamlet
    if( defined($me->{snmp}{nevalue}) && $value == $me->{snmp}{nevalue} ){
	return $me->isdown( "SNMP equal" );
    }
    return $me->isup();
}

1;
